// Player.java
// Select your personal session ID! (same for server)

import java.util.*;
import ch.aplu.tcp.*;
import ch.aplu.jgamegrid.*;

public class Player extends TcpAgent
  implements GGMouseListener
{
  private static final String VERSION = "1.3";
  private static String sessionID = "TicTacToeExt_8arewf7%&%2134";
  private static final String bridgeName = "TicTacToeServer";
  private String localName;
  private String roomName;
  private volatile boolean isMyTurn = false;
  private volatile boolean isOver = false;
  private char mySign;
  private char teammateSign;
  private Guide guide = null;
  private final Board board = new Board();
  private final Location hideLoc = new Location(-1, -1);

  public Player(final String roomName, String nickname)
  {
    super(sessionID, bridgeName);
    this.roomName = roomName;
    board.addMouseListener(this, GGMouse.lClick
      | GGMouse.move | GGMouse.leave | GGMouse.enter);
    addTcpAgentListener(new TcpAgentAdapter()
    {
      public void dataReceived(String source, int[] data)
      {
        switch (data[0])
        {
          case Command.IN_PROGRESS:
            board.setStatusText("Gameroom occupied.\nTerminating now...");
            TcpTools.delay(4000);
            System.exit(0);
            break;
          case Command.TEAMMATE_QUIT:
            board.setTitle(TcpTools.stripNickname(localName) + " in room " + roomName);
            board.setStatusText("Teammate left game room.\nWaiting for another teammate...");
            board.clear();
            isMyTurn = false;
            isOver = false;
            break;
          case Command.TEAMMATE_WAIT:
            board.setStatusText("Entered in game room.\nWaiting for teammate...");
            break;
          case Command.MARK_X:
            mySign = 'X';
            teammateSign = 'O';
            guide = new Guide('X');
            break;
          case Command.MARK_O:
            mySign = 'O';
            teammateSign = 'X';
            guide = new Guide('O');
            break;
          case Command.START_FIRST:
            TcpTools.delay(1000);   // Wait until second player got start request
            board.clear();
            board.addActor(guide, hideLoc);
            board.setStatusText("Game started.\nClick to put your mark.");
            isOver = false;
            isMyTurn = true;
            break;
          case Command.START_SECOND:
            board.clear();
            board.addActor(guide, hideLoc);
            board.setStatusText("Game started.\nWait for partner's move.");
            isOver = false;
            isMyTurn = false;
            break;
          case Command.PLAYER_NAMES:
            String playerNames = TcpTools.intAryToString(data, 1);
            board.setTitle(playerNames + " (in " + roomName + ")");
            break;
          case Command.MOVE:
            board.setStatusText("It's your move.\nClick to put your mark.");
            Location loc = new Location(data[1], data[2]);
            board.addActor(new Mark(teammateSign), loc);
            isMyTurn = true;
            break;
          case Command.WON:
            isOver = true;
            board.setStatusText("Game over. You won.\nClick in board to restart.");
            break;
          case Command.LOST:
            isOver = true;
            board.setStatusText("Game over. You lost.\nClick in board to restart.");
            break;
          case Command.TIE:
            isOver = true;
            board.setStatusText("Game over. It is a draw.\nClick in board to restart.");
            break;
        }
        if (isOver)
        {
          isMyTurn = false;
          board.removeActors(Guide.class);
          board.addActor(new Actor("sprites/gameover.gif"), new Location(1, 1));
        }
      }

      public void notifyBridgeConnection(boolean connected)
      {
        if (!connected)
        {
          board.setStatusText("Game server stopped.\nTerminating now...");
          TcpTools.delay(4000);
          System.exit(0);
        }
      }
    });
    ArrayList<String> connectList = connectToRelay(nickname);
    if (!isBridgeConnected())
    {
      board.setStatusText("Connected, but game server not found.\nTerminating now...");
      TcpTools.delay(4000);
      System.exit(0);
      return;
    }
    localName = connectList.get(0);
    board.setTitle(TcpTools.stripNickname(localName) + " in room " + roomName);
    sendCommand("", Command.REQUEST_GAME_ENTRY, TcpTools.stringToIntAry(roomName));
  }

public boolean mouseEvent(GGMouse mouse)
{
  if (isOver && mouse.getEvent() == GGMouse.lClick)
  {
    isOver = false;
    board.clear();
    board.setStatusText("Restart requested.\nWait for teammates's approval.");
    TcpTools.delay(1000);
    send("", Command.RESTART);
    return true;
  }

  if (isMyTurn)
  {
    Location loc = board.toLocation(mouse.getX(), mouse.getY());
    switch (mouse.getEvent())
    {
      case GGMouse.move:
      case GGMouse.enter:
        if (!guide.getLocation().equals(loc))
        {
          if  (board.getOneActorAt(loc, Mark.class) == null)
            guide.setLocation(loc);
          else
            guide.setLocation(hideLoc);
          board.refresh();
        }
        break;
      case GGMouse.leave:
        guide.setLocation(hideLoc);
        board.refresh();
        break;
      case GGMouse.lClick:
        if (board.getOneActorAt(loc, Mark.class) == null)
        {
          send("", Command.MOVE, loc.x, loc.y, mySign == 'O' ? 0 : 1);
          isMyTurn = false;
          board.setStatusText("Game in progress.\nWait for partner's move.");
          board.addActor(new Mark(mySign), loc);
        }
        break;
    }
  }
  return true;
}
}
